

#include <msp430x16x.h>
#include "Uart.h"

/***********************************궨*********************************/
#define UxCTL   U0CTL
#define UxRCTL  U0RCTL
#define UxTCTL  U0TCTL

#define UxBR0   U0BR0
#define UxBR1   U0BR1
#define UxMCTL  U0MCTL

#define UxRXBUF U0RXBUF
#define UxTXBUF U0TXBUF

#define UxME    U0ME
#define UxIE    U0IE
#define UxIFG   U0IFG

#define UTXEx   UTXE0
#define URXEx   URXE0

#define URXIEx  URXIE0
#define UTXIEx  UTXIE0

#define UARTxTX_VECTOR  UART0TX_VECTOR
#define UARTxRX_VECTOR  UART0RX_VECTOR

#define UARTON  P3SEL |= 0X30           // P3.4,5 = USART0 TXD/RXD
/**************************************************************************/

char TxFlag=1;
char RxFlag=0;


/****************************************************************************
*     ƣabs
*     ܣֵ
* ڲi:
* ڲint õľֵ
****************************************************************************/
int abs(int i)
{
    return (i>=0?i:(-i));
}

/***************************************************************************
*     ƣSetBaudRateRegisters
*     ܣʱ öӦĴ
* ڲ
*           clk:        ѡʱƵʣ磺32768
*           baud:             (300~115200)
* ڲ
*     : SetBaudRateRegisters(32768,9600)//ʱƵ327689600Ĳ
****************************************************************************/
void SetBaudRateRegisters(long clk,long baud)
{
    int n = clk / baud;     //
    char mSum = 0;          //mi
    int txEr0;              //ӦλΪ0ʱ
    int txEr1;              //ӦλΪ1ʱ
    char i = 0;             //ѭ
    
    UxBR1 = n >> 8;         //8λ
    UxBR0 = n & 0xff;       //8λ
    UxMCTL = 0;
    
    //ѭ ȽϴʴС UxMCTL
    for(;i < 8;i++)
    {
        txEr0 = 100 * baud * ((i + 1) * n + mSum) / clk - 100 * (i + 1);
        txEr1 = 100 * baud * ((i + 1) * n + mSum + 1) / clk - 100 * (i + 1);
        if(abs(txEr1) < abs(txEr0))
        {
            mSum++;
            UxMCTL |= (1<<i);
        }
    }
}

/****************************************************************************
*     ƣUartInit()
*     ܣʼڡòʵ
* ڲ
*           baud:             (300~115200)
            parity:     żУλ  ('n'=У  'p'=żУ  'o'=У)
            datsBits:   λλ  (78)
            stopBits:   ֹͣλλ  (12)
* ڲֵΪ1ʱɹΪ0ʾд
*     : UartInit(9600,'n',8,1) //9600bpsУ飬8λݣ1λֹͣλ
            UartInit(2400,'p',7,2) //2400bpsżУ飬7λݣ2λֹͣλ
****************************************************************************/
char UartInit(long baud,char parity,char dataBits,char stopBits)
{
    long int brclk;                 //ʷʱƵ
    
    UxCTL |= SWRST;                 //ʼ
    
    //ʱԴ
    UxTCTL &=~ (SSEL0+SSEL1);       //֮ǰʱ
    if(baud<=9600)                  //
    {
      UxTCTL |= SSEL0;              //ACLK͹
      brclk = 32768;                //ʷʱƵ=ACLK(32768)
    }
    else
    {
      UxTCTL |= SSEL1;              //SMCLK֤ٶ
      brclk = 1000000;              //ʷʱƵ=SMCLK(1MHz)
    }
    
    //------------------------ò-------------------------   
    if(baud < 300||baud > 115200)   //ʳΧ
    {
        return 0;
    }
    SetBaudRateRegisters(brclk,baud);         //òʼĴ
  
    //------------------------Уλ-------------------------  
    switch(parity)
    {
        case 'n':case'N': UxCTL &=~ PENA;               break;  //У
        case 'p':case'P': UxCTL |= PENA + PEV;          break;  //żУ
        case 'o':case'O': UxCTL |= PENA; UxCTL &=~ PEV; break;  //У  
        default :         return(0);                            //
    }
    
    //------------------------λ-------------------------    
    switch(dataBits)
    {
        case 7:case'7': UxCTL &=~ CHAR; break;      //7λ
        case 8:case'8': UxCTL |= CHAR;  break;      //8λ
        default :       return(0);                  //
    } 
    //------------------------ֹͣλ-------------------------    
    switch(stopBits)
    {
        case 1:case'1': UxCTL &= ~SPB;  break;      //1λֹͣλ
        case 2:case'2': UxCTL |= SPB;   break;      //2λֹͣλ
        default :       return(0);                  //
    }
    
    UARTON;                     //˿ʹ
    UxME |= UTXEx + URXEx;      // ʹ
    
    UCTL0 &= ~SWRST;            // Initialize USART state machine
    
    UxIE |= URXIEx + UTXIEx;    // Enable USART0 RX interrupt 
    
    return(1);                  //óɹ
}

/****************************************************************************
*     ƣUartLpm
*     ܣ/ȴУʱϵͳĲʱӸùصʡ
* ڲ
* ڲ
* ˵    : ʱӳͻע͵úӹġ
****************************************************************************/
void UartLpm()
{
    if(UxTCTL&SSEL0) LPM3;  //ACLK ʱӣLPM3(ACLK)
    else             LPM0;  //SMCLKʱӣLPM0(رSMCLK)
}

/****************************************************************************
*     ƣUartWriteChar
*     ܣ򴮿дһֽ
* ڲc:  ͵һֽ
* ڲ
* ˵    : ڵȴϵĹУCPU
****************************************************************************/
void UartWriteChar(char c)
{ 
    while (TxFlag==0) UartLpm();  // ȴһֽڷ꣬
    TxFlag=0;                     //
    UxTXBUF=c;
}

/****************************************************************************
*     ƣUartReadChar
*     ܣӴڶȡ1ֽ
* ڲ  
* ڲյһֽ
* ˵    : ûݣһֱȴȴУCPU
****************************************************************************/
char UartReadChar()
{ 
    while (RxFlag==0) UartLpm(); // յһֽ?
    RxFlag=0;
    return(UxRXBUF);
}

/****************************************************************************
*     ƣUartWriteStr
*     ܣ򴮿дһַ
* ڲsҪдַ  
* ڲ
* ˵    : ûе NULL дַ
****************************************************************************/
void UartWriteStr(char *s)
{
    while(*s)
    {
        UartWriteChar(*s++);
    }
}

/****************************************************************************
*     ƣUartRx
*     ܣڽжϣÿյ1ֽڻᷢһж
****************************************************************************/
#pragma vector=UARTxRX_VECTOR
__interrupt void UartRx()
{
    RxFlag=1;
    __low_power_mode_off_on_exit();
}

/****************************************************************************
*     ƣUartTx
*     ܣڷжϣÿ1ֽڻᷢһж
****************************************************************************/
#pragma vector=UARTxTX_VECTOR
__interrupt void UartTx ()
{
    TxFlag=1;
    __low_power_mode_off_on_exit();
}
